﻿--[[                                      CurrencyMonitor

This program watches a player's justice points, valor points, conquest points, and honor points
and flashes a raid warning with optional raid warning sound when any point gain above the minimum of 
50 makes the points total exceed its player-defined threshold. 

               *** CurrencyMonitor is Copyright 2011 by Malavice of Mal'Ganis US ***               

Please do not distribute altered copies of this add-on because it can make maintaining it a 
nightmare. If you want to see a feature built into it, please add a comment on the official 
download pages on curse.com and wowinterface.com.

For now, I will take bug reports and feature requests directly in the comments sections of the 
official release channels. Until such a time that I believe I need a ticket tracking system (which
hopefully never happens), I will simply answer bug reports/feature requests in comments sections.

---------------
Version History
---------------

10/27/2014 - V.2.0.2b	 Version bump for 6.0 release.
02/07/2014 - V.2.0.2a    Interface version bump and corrected matching string to fix lua errors when
						 awarded bonus Valor Points from Heroic Scenarios. Dropped minimum point
						 to 30.
08/26/2012 - V.2.0.2     Restructured the saved variables table to prevent errors on load when 
						 currencies were not tracked.
07/11/2012 - V.2.0.1	 CurrencyMonitor wasn't flashing any alerts, but at least the GUI worked. CM
						 will again alert users that their currency is over the threshold.
07/08/2012 - V.2.0       Created the first in-game GUI. Bugs likely. Added slash command to hide or
						 show the GUI. 
07/02/2012 - V.1.2.3	 Removed an unnecessary registered event. Matched default values to 
						 description. Flipped the change log. Moved variables around in the file to
						 make maintaining it just a bit easier.
02/24/2012 - V.1.2.2	 Fixed a problem with the currency warning not updating to reflect the
						 current amount of points when points are awarded. Minor .toc file fix.
12/15/2011 - V.1.2.1	 Changed some variables to local to avoid polluting the global environment.
12/09/2011 - V.1.2       Spam bug fix. Also warns of only the type of currency that triggered the 
					     warning. Considerable code rewrite for less redundancy.
12/05/2011 - V.1.1       Removed BG spamminess through smart points announcement.
12/02/2011 - V.1.0       Now firing on the correct event.					 
12/02/2011 - V.0.1(BETA) (Initial release)

-----
To-Do
-----

Nap. Damn I love naps. Future plans might include a configurable minToWarn variable, but 50 seems to
be just right so I probably won't mess with it. 

--- Credits ---

Credit goes to Wowhead user eidyia for the idea and additional input that helped hone this add-on's
functionality. Specifically, thank eidyia for the version 1.1 removal of BG spam and the 1.2 removal
of buggy spam!

Seerah and Dridzt of wowinterface.com also deserve some major thanks for helping me iron out the 
wrinkles in saving my variables in a table between sessions. I was just about to give up on the idea
but it was nice to see their encouragements. Thanks you two!

--]]

----------------- Know what you're doing before you edit anything below this line. -----------------

-- This variable will be configurable in the future if there is at least a little demand for it
local minToWarn = 30

-- Add slash command configuration menu
SLASH_CURRENCYMONITOR1, SLASH_CURRENCYMONITOR2 = "/cm", "/currencymonitor";

function SlashCmdList.CURRENCYMONITOR(msg, editbox)
	CM_ToggleGUI()
end

-- Table to store currency IDs for querying the server
local currencies = {
	"396", -- Valor points
	"395", -- Justice points
	"390", -- Conquest points
	"392"  -- Honor points
}

-- Table to store currency names for display to the player
-- Working to remove this table
local allCurrency = {"Valor","Justice","Conquest","Honor",}

-- Tables to transfer field input into the saved variables fields
local currencyThresholds = {}

-- An empty table that gets used more often than a $5 whore
local currentAmount = {}

-- Update all thresholds with the edit box values when the Save button is pressed
function CM_UpdateAll(v,j,c,h,vt,jt,ct,ht)

	currencyThresholds = {v:GetText(), j:GetText(), c:GetText(), h:GetText(),}	
	
	for i = 1, 4 do
		CM_db[i]["threshold"] = currencyThresholds[i]
	end

	-- I'm sure I could iterate over this instead of what I did here, but meh
	if vt:GetChecked() then CM_db[1]["tracked"] = "tracked" else CM_db[1]["tracked"] = "not tracked" end
	if jt:GetChecked() then CM_db[2]["tracked"] = "tracked" else CM_db[2]["tracked"] = "not tracked" end
	if ct:GetChecked() then CM_db[3]["tracked"] = "tracked" else CM_db[3]["tracked"] = "not tracked" end
	if ht:GetChecked() then CM_db[4]["tracked"] = "tracked" else CM_db[4]["tracked"] = "not tracked" end 

	CM_ToggleGUI()
end

-- Toggle playing the Raid Warning sound or not depending on the button's GetChecked() return
function CM_updateRWSound(self)
	
	if self:GetChecked() then
		CM_warningSound = true
		PlaySoundFile("Sound\\Interface\\RaidWarning.wav")
		RaidNotice_AddMessage(RaidWarningFrame, "Raid Warning sound is now ENABLED!", ChatTypeInfo["RAID_WARNING"])
	else
		CM_warningSound = false
		RaidNotice_AddMessage(RaidWarningFrame, "Raid Warning sound is now DISABLED!", ChatTypeInfo["RAID_WARNING"])
	end
end

-- Loop through and get the current amount of the four currencies we care about
local function getAllCurrencies()

	for i = 1, 4 do
		local _, amountNow = GetCurrencyInfo(currencies[i])
		table.insert(currentAmount, amountNow)
	end
end

-- Adds the warning message to the raid warning frame 
local function printOutput(triggerCurrency)

	-- If our trigger currency is in the list of currencies the add-on is set to recognize, play the
	-- raid warning sound if enabled and flash the add-on's alert to the user
	for i = 1, 4 do
		if triggerCurrency == CM_db[i]["pointType"] then
			if CM_warningSound then PlaySoundFile("Sound\\Interface\\RaidWarning.wav") end
			RaidNotice_AddMessage(RaidWarningFrame, "You have "..currentAmount[i].." "..CM_db[i]["pointType"].." points!", ChatTypeInfo["RAID_WARNING"])
		end
	end
end

-- The functions to run when our registered events fire
function CurrencyMonitor_OnEvent(self, event, arg1)
	
	-- If we just gained currency and it's greater than our minimum, then check to see if (a) it's
	-- one we're tracking and (b) our new total is greater than or equal to our threshold
	if event == "CHAT_MSG_CURRENCY" and tonumber(string.match(arg1, '%d+')) >= minToWarn then
		getAllCurrencies()
		for i = 1, 4 do
			local triggerCurrency = string.match(arg1, CM_db[i]["pointType"])
			if (triggerCurrency ~= nil and CM_db[i]["tracked"] == "tracked" and currentAmount[i] >= tonumber(CM_db[i]["threshold"])) then
				printOutput(triggerCurrency)
			end
		end
		-- Clear the temporary table
		currentAmount = nil
		currentAmount = {}
	end
	
	if (event == "ADDON_LOADED" and arg1 == "CurrencyMonitor") then
		-- If we don't have saved variables, then load some default values
		if not CM_db then
			CM_db = {}
			CM_db = {
				{["pointType"] = "Valor", ["tracked"] = "tracked", ["threshold"] = 2000,},
				{["pointType"] = "Justice", ["tracked"] = "tracked", ["threshold"] = 2000,},
				{["pointType"] = "Conquest", ["tracked"] = "tracked", ["threshold"] = 2000,},
				{["pointType"] = "Honor", ["tracked"] = "tracked", ["threshold"] = 2000},
			}
		end
		
		-- Load GUI with check boxes marked according to settings
		local buttons = {CM_ValorTrack,CM_JusticeTrack,CM_ConquestTrack,CM_HonorTrack}
		for i = 1, 4 do
			if CM_db[i]["tracked"] == "tracked" then
				buttons[i]:SetChecked(true)
			else
				buttons[i]:SetChecked(false)
			end
		end
		
		-- Set the edit boxes' default values
		CM_ValorEdit:SetText(CM_db[1]["threshold"])
		CM_JusticeEdit:SetText(CM_db[2]["threshold"])
		CM_ConquestEdit:SetText(CM_db[3]["threshold"])
		CM_HonorEdit:SetText(CM_db[4]["threshold"])
		
		-- Load the Raid Warning sound check box according to settings
		RWSoundSelect:SetChecked(CM_warningSound)
	end
end

-- Hide the GUI if it's shown, or show it if it's hidden
function CM_ToggleGUI(self)

	if CurrencyMonitorFrame:IsVisible(self) then
		CurrencyMonitorFrame:Hide(self)
	else
		CurrencyMonitorFrame:Show(self)
	end
end